//
// Webpack alias "variables" points to this file.
// So you can import it in your app's *.vue files
// inside the <style> tag like below.
//
// NOTICE that you need lang="styl"
//
// <style lang="styl">
// @import '~variables'
// ........
// </style>


// First we load app's Stylus variables
@import 'app.variables'

// Then we load Quasar Stylus variables.
// Any variables defined in "app.variables.styl"
// will override Quasar's ones.
//
// NOTICE that we only import Core Quasar Variables
// like colors, media breakpoints, and so.
// No component variable will be included.
@import '~quasar/dist/core.variables'
